@@ -10,6 +10,7 @@ package postgres
 import (
 	"context"
 	"database/sql"
+	"errors"
 	"fmt"
 	"io"
 	"strings"
@@ -19,10 +20,32 @@ import (
 	"github.com/xo/usql/drivers"
 	"github.com/xo/usql/drivers/metadata"
 	pgmeta "github.com/xo/usql/drivers/metadata/postgres"
+	"github.com/xo/usql/env"
 	"github.com/xo/usql/text"
 )
 
 func init() {
+	openConn := func(stdout, stderr func() io.Writer, dsn string) (*sql.DB, error) {
+		conn, err := pq.NewConnector(dsn)
+		if err != nil {
+			return nil, err
+		}
+		noticeConn := pq.ConnectorWithNoticeHandler(conn, func(notice *pq.Error) {
+			out := stderr()
+			fmt.Fprintln(out, notice.Severity+": ", notice.Message)
+			if notice.Hint != "" {
+				fmt.Fprintln(out, "HINT: ", notice.Hint)
+			}
+		})
+		notificationConn := pq.ConnectorWithNotificationHandler(noticeConn, func(notification *pq.Notification) {
+			var payload string
+			if notification.Extra != "" {
+				payload = fmt.Sprintf(text.NotificationPayload, notification.Extra)
+			}
+			fmt.Fprintln(stdout(), fmt.Sprintf(text.NotificationReceived, notification.Channel, payload, notification.BePid))
+		})
+		return sql.OpenDB(notificationConn), nil
+	}
 	drivers.Register("postgres", drivers.Driver{
 		Name:                   "pq",
 		AllowDollar:            true,
@@ -33,27 +56,27 @@ func init() {
 				drivers.ForceQueryParameters([]string{"sslmode", "disable"})(u)
 			}
 		},
-		Open: func(u *dburl.URL, stdout, stderr func() io.Writer) (func(string, string) (*sql.DB, error), error) {
-			return func(typ, dsn string) (*sql.DB, error) {
-				conn, err := pq.NewConnector(dsn)
+		Open: func(ctx context.Context, u *dburl.URL, stdout, stderr func() io.Writer) (func(string, string) (*sql.DB, error), error) {
+			return func(_, dsn string) (*sql.DB, error) {
+				conn, err := openConn(stdout, stderr, dsn)
 				if err != nil {
 					return nil, err
 				}
-				noticeConn := pq.ConnectorWithNoticeHandler(conn, func(notice *pq.Error) {
-					out := stderr()
-					fmt.Fprintln(out, notice.Severity+": ", notice.Message)
-					if notice.Hint != "" {
-						fmt.Fprintln(out, "HINT: ", notice.Hint)
+				// special retry handling case, since there's no lib/pq retry mode
+				if env.Get("SSLMODE") == "retry" && !u.Query().Has("sslmode") {
+					switch err = conn.PingContext(ctx); {
+					case errors.Is(err, pq.ErrSSLNotSupported):
+						s := "sslmode=disable " + dsn
+						conn, err = openConn(stdout, stderr, s)
+						if err != nil {
+							return nil, err
+						}
+						u.DSN = s
+					case err != nil:
+						return nil, err
 					}
-				})
-				notificationConn := pq.ConnectorWithNotificationHandler(noticeConn, func(notification *pq.Notification) {
-					var payload string
-					if notification.Extra != "" {
-						payload = fmt.Sprintf(text.NotificationPayload, notification.Extra)
-					}
-					fmt.Fprintln(stdout(), fmt.Sprintf(text.NotificationReceived, notification.Channel, payload, notification.BePid))
-				})
-				return sql.OpenDB(notificationConn), nil
+				}
+				return conn, nil
 			}, nil
 		},
 		Version: func(ctx context.Context, db drivers.DB) (string, error) {
